'use client';

import { ChatContent } from './ChatContent';

export default function ChatPage() {
  return (
    <div className="flex flex-col h-screen bg-background">
      <ChatContent />
    </div>
  );
}
